@@ -31,7 +31,7 @@ except ImportError:
         sys.path.insert(0, cmd_subfolder)
     import paho.mqtt.client as mqtt
 
-def on_connect(mqttc, obj, rc):
+def on_connect(mqttc, obj, flags, rc):
     print("rc: "+str(rc))
 
 def on_message(mqttc, obj, msg):
@@ -57,8 +57,8 @@ mqttc.on_publish = on_publish
 mqttc.on_subscribe = on_subscribe
 # Uncomment to enable debug messages
 #mqttc.on_log = on_log
-mqttc.connect("localhost", 1883, 60)
-mqttc.subscribe("#", 2)
+mqttc.connect("m2m.eclipse.org", 1883, 60)
+mqttc.subscribe("$SYS/#", 0)
 
 
 mqttc.loop_forever()
