@@ -100,11 +100,16 @@ mqtt_cs_disconnecting = 2
 mqtt_cs_connect_async = 3
 
 # Message state
-mqtt_ms_invalid = 0,
-mqtt_ms_wait_puback = 1
-mqtt_ms_wait_pubrec = 2
-mqtt_ms_wait_pubrel = 3
-mqtt_ms_wait_pubcomp = 4
+mqtt_ms_invalid = 0
+mqtt_ms_publish= 1
+mqtt_ms_wait_for_puback = 2
+mqtt_ms_wait_for_pubrec = 3
+mqtt_ms_resend_pubrel = 4
+mqtt_ms_wait_for_pubrel = 5
+mqtt_ms_resend_pubcomp = 6
+mqtt_ms_wait_for_pubcomp = 7
+mqtt_ms_send_pubrec = 8
+mqtt_ms_queued = 9
 
 # Error values
 MQTT_ERR_AGAIN = -1
@@ -252,7 +257,7 @@ def _socketpair_compat():
     """TCP/IP socketpair including Windows support"""
     listensock = socket.socket(socket.AF_INET, socket.SOCK_STREAM, socket.IPPROTO_IP)
     listensock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
-    listensock.bind(("localhost", 0))
+    listensock.bind(("127.0.0.1", 0))
     listensock.listen(1)
 
     iface, port = listensock.getsockname()
@@ -382,7 +387,7 @@ class Client(object):
       MQTT_LOG_ERR, and MQTT_LOG_DEBUG. The message itself is in buf.
 
     """
-    def __init__(self, client_id="", clean_session=True, userdata=None, protocol=MQTTv311):
+    def __init__(self, client_id="", clean_session=True, userdata=None, protocol=MQTTv31):
         """client_id is the unique client id string used when connecting to the
         broker. If client_id is zero length or None, then one will be randomly
         generated. In this case, clean_session must be True. If this is not the
@@ -400,6 +405,13 @@ class Client(object):
         userdata is user defined data of any type that is passed as the "userdata"
         parameter to callbacks. It may be updated at a later point with the
         user_data_set() function.
+
+        The protocol argument allows explicit setting of the MQTT version to
+        use for this client. Can be paho.mqtt.client.MQTTv311 (v3.1.1) or
+        paho.mqtt.client.MQTTv31 (v3.1), with the default being v3.1. If the
+        broker reports that the client connected with an invalid protocol
+        version, the client will automatically attempt to reconnect using v3.1
+        instead.
         """
         if not clean_session and (client_id == "" or client_id is None):
             raise ValueError('A client id must be provided if clean session is False.')
@@ -717,16 +729,16 @@ class Client(object):
 
         try:
             if (sys.version_info[0] == 2 and sys.version_info[1] < 7) or (sys.version_info[0] == 3 and sys.version_info[1] < 2):
-                self._sock = socket.create_connection((self._host, self._port))
+                sock = socket.create_connection((self._host, self._port))
             else:
-                self._sock = socket.create_connection((self._host, self._port), source_address=(self._bind_address, 0))
+                sock = socket.create_connection((self._host, self._port), source_address=(self._bind_address, 0))
         except socket.error as err:
             if err.errno != errno.EINPROGRESS and err.errno != errno.EWOULDBLOCK and err.errno != EAGAIN:
                 raise
 
         if self._tls_ca_certs is not None:
             self._ssl = ssl.wrap_socket(
-                self._sock,
+                sock,
                 certfile=self._tls_certfile,
                 keyfile=self._tls_keyfile,
                 ca_certs=self._tls_ca_certs,
@@ -740,6 +752,7 @@ class Client(object):
                 else:
                     ssl.match_hostname(self._ssl.getpeercert(), self._host)
 
+        self._sock = sock
         self._sock.setblocking(0)
 
         return self._send_connect(self._keepalive, self._clean_session)
@@ -787,6 +800,12 @@ class Client(object):
         except TypeError:
             # Socket isn't correct type, in likelihood connection is lost
             return MQTT_ERR_CONN_LOST
+        except ValueError:
+            # Can occur if we just reconnected but rlist/wlist contain a -1 for
+            # some reason.
+            return MQTT_ERR_CONN_LOST
+        except:
+            return MQTT_ERR_UNKNOWN
 
         if self.socket() in socklist[0]:
             rc = self.loop_read(max_packets)
@@ -842,6 +861,8 @@ class Client(object):
             raise ValueError('Invalid QoS level.')
         if isinstance(payload, str) or isinstance(payload, bytearray):
             local_payload = payload
+        elif sys.version_info[0] < 3 and isinstance(payload, unicode):
+            local_payload = payload
         elif isinstance(payload, int) or isinstance(payload, float):
             local_payload = str(payload)
         elif payload is None:
@@ -880,15 +901,24 @@ class Client(object):
             if self._max_inflight_messages == 0 or self._inflight_messages < self._max_inflight_messages:
                 self._inflight_messages = self._inflight_messages+1
                 if qos == 1:
-                    message.state = mqtt_ms_wait_puback
+                    message.state = mqtt_ms_wait_for_puback
                 elif qos == 2:
-                    message.state = mqtt_ms_wait_pubrec
+                    message.state = mqtt_ms_wait_for_pubrec
                 self._out_message_mutex.release()
 
                 rc = self._send_publish(message.mid, message.topic, message.payload, message.qos, message.retain, message.dup)
+
+                # remove from inflight messages so it will be send after a connection is made
+                if rc is MQTT_ERR_NO_CONN:
+                    with self._out_message_mutex:
+                        self._inflight_messages -= 1
+                        message.state = mqtt_ms_publish
+
                 return (rc, local_mid)
-            self._out_message_mutex.release()
-            return (MQTT_ERR_SUCCESS, local_mid)
+            else:
+                message.state = mqtt_ms_queued;
+                self._out_message_mutex.release()
+                return (MQTT_ERR_SUCCESS, local_mid)
 
     def username_pw_set(self, username, password=None):
         """Set a username and optionally a password for broker authentication.
@@ -1060,7 +1090,7 @@ class Client(object):
         if self._sock is None and self._ssl is None:
             return MQTT_ERR_NO_CONN
 
-        max_packets = len(self._out_messages) + len(self._in_messages)
+        max_packets = len(self._out_packet) + 1
         if max_packets < 1:
             max_packets = 1
 
@@ -1194,17 +1224,36 @@ class Client(object):
         else:
             return self._sock
 
-    def loop_forever(self, timeout=1.0, max_packets=1):
+    def loop_forever(self, timeout=1.0, max_packets=1, retry_first_connection=False):
         """This function call loop() for you in an infinite blocking loop. It
         is useful for the case where you only want to run the MQTT client loop
         in your program.
 
         loop_forever() will handle reconnecting for you. If you call
-        disconnect() in a callback it will return."""
+        disconnect() in a callback it will return.
+
+
+        timeout: The time in seconds to wait for incoming/outgoing network
+          traffic before timing out and returning.
+        max_packets: Not currently used.
+        retry_first_connection: Should the first connection attempt be retried on failure.
+
+        Raises socket.error on first connection failures unless retry_first_connection=True
+        """
 
         run = True
-        if self._state == mqtt_cs_connect_async:
-            self.reconnect()
+
+        while run:
+            if self._state == mqtt_cs_connect_async:
+                try:
+                    self.reconnect()
+                except socket.error:
+                    if not retry_first_connection:
+                        raise
+                    self._easy_log(MQTT_LOG_DEBUG, "Connection failed, retrying")
+                    time.sleep(1)
+            else:
+                break
 
         while run:
             rc = MQTT_ERR_SUCCESS
@@ -1224,7 +1273,7 @@ class Client(object):
                     run = False
 
             self._state_mutex.acquire()
-            if self._state == mqtt_cs_disconnecting:
+            if self._state == mqtt_cs_disconnecting or run is False or self._thread_terminate is True:
                 run = False
                 self._state_mutex.release()
             else:
@@ -1232,7 +1281,7 @@ class Client(object):
                 time.sleep(1)
 
                 self._state_mutex.acquire()
-                if self._state == mqtt_cs_disconnecting:
+                if self._state == mqtt_cs_disconnecting or run is False or self._thread_terminate is True:
                     run = False
                     self._state_mutex.release()
                 else:
@@ -1252,6 +1301,7 @@ class Client(object):
         if self._thread is not None:
             return MQTT_ERR_INVAL
 
+        self._thread_terminate = False
         self._thread = threading.Thread(target=self._thread_main)
         self._thread.daemon = True
         self._thread.start()
@@ -1656,7 +1706,7 @@ class Client(object):
             remaining_length = remaining_length + 2
 
         self._pack_remaining_length(packet, remaining_length)
-        self._pack_str16(packet, utopic)
+        self._pack_str16(packet, topic)
 
         if qos > 0:
             # For message id
@@ -1790,15 +1840,15 @@ class Client(object):
         now = time.time()
         for m in messages:
             if m.timestamp + self._message_retry < now:
-                if m.state == mqtt_ms_wait_puback or m.state == mqtt_ms_wait_pubrec:
+                if m.state == mqtt_ms_wait_for_puback or m.state == mqtt_ms_wait_for_pubrec:
                     m.timestamp = now
                     m.dup = True
                     self._send_publish(m.mid, m.topic, m.payload, m.qos, m.retain, m.dup)
-                elif m.state == mqtt_ms_wait_pubrel:
+                elif m.state == mqtt_ms_wait_for_pubrel:
                     m.timestamp = now
                     m.dup = True
                     self._send_pubrec(m.mid)
-                elif m.state == mqtt_ms_wait_pubcomp:
+                elif m.state == mqtt_ms_wait_for_pubcomp:
                     m.timestamp = now
                     m.dup = True
                     self._send_pubrel(m.mid, True)
@@ -1810,16 +1860,28 @@ class Client(object):
 
     def _messages_reconnect_reset_out(self):
         self._out_message_mutex.acquire()
+        self._inflight_messages = 0
         for m in self._out_messages:
             m.timestamp = 0
             if self._max_inflight_messages == 0 or self._inflight_messages < self._max_inflight_messages:
-                if m.qos == 1:
-                    m.state = mqtt_ms_wait_puback
+                if m.qos == 0:
+                    m.state = mqtt_ms_publish
+                elif m.qos == 1:
+                    #self._inflight_messages = self._inflight_messages + 1
+                    if m.state == mqtt_ms_wait_for_puback:
+                        m.dup = True
+                    m.state = mqtt_ms_publish
                 elif m.qos == 2:
-                    # Preserve current state
-                    pass
+                    #self._inflight_messages = self._inflight_messages + 1
+                    if m.state == mqtt_ms_wait_for_pubcomp:
+                        m.state = mqtt_ms_resend_pubrel
+                        m.dup = True
+                    else:
+                        if m.state == mqtt_ms_wait_for_pubrec:
+                            m.dup = True
+                        m.state = mqtt_ms_publish
             else:
-                m.state = mqtt_ms_invalid
+                m.state = mqtt_ms_queued
         self._out_message_mutex.release()
 
     def _messages_reconnect_reset_in(self):
@@ -1949,7 +2011,54 @@ class Client(object):
             self._in_callback = False
         self._callback_mutex.release()
         if result == 0:
-            return MQTT_ERR_SUCCESS
+            rc = 0
+            self._out_message_mutex.acquire()
+            for m in self._out_messages:
+                m.timestamp = time.time()
+                if m.state == mqtt_ms_queued:
+                    self.loop_write() # Process outgoing messages that have just been queued up
+                    self._out_message_mutex.release()
+                    return MQTT_ERR_SUCCESS
+
+                if m.qos == 0:
+                    self._in_callback = True # Don't call loop_write after _send_publish()
+                    rc = self._send_publish(m.mid, m.topic, m.payload, m.qos, m.retain, m.dup)
+                    self._in_callback = False
+                    if rc != 0:
+                        self._out_message_mutex.release()
+                        return rc
+                elif m.qos == 1:
+                    if m.state == mqtt_ms_publish:
+                        self._inflight_messages = self._inflight_messages + 1
+                        m.state = mqtt_ms_wait_for_puback
+                        self._in_callback = True # Don't call loop_write after _send_publish()
+                        rc = self._send_publish(m.mid, m.topic, m.payload, m.qos, m.retain, m.dup)
+                        self._in_callback = False
+                        if rc != 0:
+                            self._out_message_mutex.release()
+                            return rc
+                elif m.qos == 2:
+                    if m.state == mqtt_ms_publish:
+                        self._inflight_messages = self._inflight_messages + 1
+                        m.state = mqtt_ms_wait_for_pubrec
+                        self._in_callback = True # Don't call loop_write after _send_publish()
+                        rc = self._send_publish(m.mid, m.topic, m.payload, m.qos, m.retain, m.dup)
+                        self._in_callback = False
+                        if rc != 0:
+                            self._out_message_mutex.release()
+                            return rc
+                    elif m.state == mqtt_ms_resend_pubrel:
+                        self._inflight_messages = self._inflight_messages + 1
+                        m.state = mqtt_ms_wait_for_pubcomp
+                        self._in_callback = True # Don't call loop_write after _send_pubrel()
+                        rc = self._send_pubrel(m.mid, m.dup)
+                        self._in_callback = False
+                        if rc != 0:
+                            self._out_message_mutex.release()
+                            return rc
+                self.loop_write() # Process outgoing messages that have just been queued up
+            self._out_message_mutex.release()
+            return rc
         elif result > 0 and result < 6:
             return MQTT_ERR_CONN_REFUSED
         else:
@@ -2014,7 +2123,7 @@ class Client(object):
             return rc
         elif message.qos == 2:
             rc = self._send_pubrec(message.mid)
-            message.state = mqtt_ms_wait_pubrel
+            message.state = mqtt_ms_wait_for_pubrel
             self._in_message_mutex.acquire()
             self._in_messages.append(message)
             self._in_message_mutex.release()
@@ -2061,12 +2170,12 @@ class Client(object):
         # Dont lock message_mutex here
         for m in self._out_messages:
             if self._inflight_messages < self._max_inflight_messages:
-                if m.qos > 0 and m.state == mqtt_ms_invalid:
+                if m.qos > 0 and m.state == mqtt_ms_queued:
                     self._inflight_messages = self._inflight_messages + 1
                     if m.qos == 1:
-                        m.state = mqtt_ms_wait_puback
+                        m.state = mqtt_ms_wait_for_puback
                     elif m.qos == 2:
-                        m.state = mqtt_ms_wait_pubrec
+                        m.state = mqtt_ms_wait_for_pubrec
                     rc = self._send_publish(m.mid, m.topic, m.payload, m.qos, m.retain, m.dup)
                     if rc != 0:
                         return rc
@@ -2086,7 +2195,7 @@ class Client(object):
         self._out_message_mutex.acquire()
         for m in self._out_messages:
             if m.mid == mid:
-                m.state = mqtt_ms_wait_pubcomp
+                m.state = mqtt_ms_wait_for_pubcomp
                 m.timestamp = time.time()
                 self._out_message_mutex.release()
                 return self._send_pubrel(mid, False)
@@ -2168,8 +2277,6 @@ class Client(object):
         self._callback_mutex.release()
 
     def _thread_main(self):
-        run = True
-        self._thread_terminate = False
         self._state_mutex.acquire()
         if self._state == mqtt_cs_connect_async:
             self._state_mutex.release()
@@ -2179,6 +2286,23 @@ class Client(object):
 
         self.loop_forever()
 
+    def _host_matches_cert(self, host, cert_host):
+        if cert_host[0:2] == "*.":
+            if cert_host.count("*") != 1:
+                return False
+
+            host_match = host.split(".", 1)[1]
+            cert_match = cert_host.split(".", 1)[1]
+            if host_match == cert_match:
+                return True
+            else:
+                return False
+        else:
+            if host == cert_host:
+                return True
+            else:
+                return False
+
     def _tls_match_hostname(self):
         cert = self._ssl.getpeercert()
         san = cert.get('subjectAltName')
@@ -2187,7 +2311,7 @@ class Client(object):
             for (key, value) in san:
                 if key == 'DNS':
                     have_san_dns = True
-                    if value.lower() == self._host.lower():
+                    if self._host_matches_cert(self._host.lower(), value.lower()) == True:
                         return
                 if key == 'IP Address':
                     have_san_dns = True
@@ -2201,7 +2325,7 @@ class Client(object):
         if subject:
             for ((key, value),) in subject:
                 if key == 'commonName':
-                    if value.lower() == self._host.lower():
+                    if self._host_matches_cert(self._host.lower(), value.lower()) == True:
                         return
 
         raise ssl.SSLError('Certificate subject does not match remote hostname.')
